@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/bin/env python
 """
 mraptor.py - MacroRaptor
 
@@ -23,7 +23,7 @@ http://www.decalage.info/python/oletools
 
 # === LICENSE ==================================================================
 
-# MacroRaptor is copyright (c) 2016-2018 Philippe Lagadec (http://www.decalage.info)
+# MacroRaptor is copyright (c) 2016-2021 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -58,8 +58,12 @@ http://www.decalage.info/python/oletools
 # 2016-12-21 v0.51 PL: - added more ActiveX macro triggers
 # 2017-03-08       PL: - fixed absolute imports
 # 2018-05-25 v0.53 PL: - added Word/PowerPoint 2007+ XML (aka Flat OPC) issue #283
+# 2019-04-04 v0.54 PL: - added ExecuteExcel4Macro, ShellExecuteA, XLM keywords
+# 2019-11-06 v0.55 PL: - added SetTimer
+# 2020-04-20 v0.56 PL: - added keywords RUN and CALL for XLM macros (issue #562)
+# 2021-04-14       PL: - added Workbook_BeforeClose (issue #518)
 
-__version__ = '0.53'
+__version__ = '0.56.2'
 
 #------------------------------------------------------------------------------
 # TODO:
@@ -67,7 +71,7 @@ __version__ = '0.53'
 
 #--- IMPORTS ------------------------------------------------------------------
 
-import sys, logging, optparse, re, os
+import sys, optparse, re, os
 
 # IMPORTANT: it should be possible to run oletools directly as scripts
 # in any directory without installing them with pip or setup.py.
@@ -86,11 +90,12 @@ from oletools.thirdparty.tablestream import tablestream
 
 from oletools import olevba
 from oletools.olevba import TYPE2TAG
+from oletools.common.log_helper import log_helper
 
 # === LOGGING =================================================================
 
 # a global logger object used for debugging:
-log = olevba.get_logger('mraptor')
+log = log_helper.get_or_create_silent_logger('mraptor')
 
 
 #--- CONSTANTS ----------------------------------------------------------------
@@ -113,26 +118,27 @@ MSG_ISSUES = 'Please report this issue on %s' % URL_ISSUES
 # TODO: check if line also contains Sub or Function
 re_autoexec = re.compile(r'(?i)\b(?:Auto(?:Exec|_?Open|_?Close|Exit|New)' +
                          r'|Document(?:_?Open|_Close|_?BeforeClose|Change|_New)' +
-                         r'|NewDocument|Workbook(?:_Open|_Activate|_Close)' +
+                         r'|NewDocument|Workbook(?:_Open|_Activate|_Close|_BeforeClose)' +
                          r'|\w+_(?:Painted|Painting|GotFocus|LostFocus|MouseHover' +
                          r'|Layout|Click|Change|Resize|BeforeNavigate2|BeforeScriptExecute' +
                          r'|DocumentComplete|DownloadBegin|DownloadComplete|FileDownload' +
                          r'|NavigateComplete2|NavigateError|ProgressChange|PropertyChange' +
                          r'|SetSecureLockIcon|StatusTextChange|TitleChange|MouseMove' +
-                         r'|MouseEnter|MouseLeave|))\b')
+                         r'|MouseEnter|MouseLeave|OnConnecting))|Auto_Ope\b')
+# TODO: "Auto_Ope" is temporarily here because of a bug in plugin_biff, which misses the last byte in "Auto_Open"...
 
 # MS-VBAL 5.4.5.1 Open Statement:
 RE_OPEN_WRITE = r'(?:\bOpen\b[^\n]+\b(?:Write|Append|Binary|Output|Random)\b)'
 
 re_write = re.compile(r'(?i)\b(?:FileCopy|CopyFile|Kill|CreateTextFile|'
-    + r'VirtualAlloc|RtlMoveMemory|URLDownloadToFileA?|AltStartupPath|'
+    + r'VirtualAlloc|RtlMoveMemory|URLDownloadToFileA?|AltStartupPath|WriteProcessMemory|'
     + r'ADODB\.Stream|WriteText|SaveToFile|SaveAs|SaveAsRTF|FileSaveAs|MkDir|RmDir|SaveSetting|SetAttr)\b|' + RE_OPEN_WRITE)
 
 # MS-VBAL 5.2.3.5 External Procedure Declaration
 RE_DECLARE_LIB = r'(?:\bDeclare\b[^\n]+\bLib\b)'
 
-re_execute = re.compile(r'(?i)\b(?:Shell|CreateObject|GetObject|SendKeys|'
-    + r'MacScript|FollowHyperlink|CreateThread|ShellExecute)\b|' + RE_DECLARE_LIB)
+re_execute = re.compile(r'(?i)\b(?:Shell|CreateObject|GetObject|SendKeys|RUN|CALL|'
+    + r'MacScript|FollowHyperlink|CreateThread|ShellExecuteA?|ExecuteExcel4Macro|EXEC|REGISTER|SetTimer)\b|' + RE_DECLARE_LIB)
 
 
 # === CLASSES =================================================================
@@ -225,15 +231,7 @@ def main():
     """
     Main function, called when olevba is run from the command line
     """
-    global log
     DEFAULT_LOG_LEVEL = "warning" # Default log level
-    LOG_LEVELS = {
-        'debug':    logging.DEBUG,
-        'info':     logging.INFO,
-        'warning':  logging.WARNING,
-        'error':    logging.ERROR,
-        'critical': logging.CRITICAL
-        }
 
     usage = 'usage: mraptor [options] <filename> [filename2 ...]'
     parser = optparse.OptionParser(usage=usage)
@@ -267,9 +265,9 @@ def main():
     print('MacroRaptor %s - http://decalage.info/python/oletools' % __version__)
     print('This is work in progress, please report issues at %s' % URL_ISSUES)
 
-    logging.basicConfig(level=LOG_LEVELS[options.loglevel], format='%(levelname)-8s %(message)s')
+    log_helper.enable_logging(level=options.loglevel)
     # enable logging in the modules:
-    log.setLevel(logging.NOTSET)
+    olevba.enable_logging()
 
     t = tablestream.TableStream(style=tablestream.TableStyleSlim,
             header_row=['Result', 'Flags', 'Type', 'File'],
@@ -314,8 +312,7 @@ def main():
             if vba_parser.detect_vba_macros():
                 vba_code_all_modules = ''
                 try:
-                    for (subfilename, stream_path, vba_filename, vba_code) in vba_parser.extract_all_macros():
-                        vba_code_all_modules += vba_code + '\n'
+                    vba_code_all_modules = vba_parser.get_vba_code_all_modules()
                 except Exception as e:
                     # log.error('Error when parsing VBA macros from file %r' % full_name)
                     result = Result_Error
@@ -342,6 +339,7 @@ def main():
             global_result = result
             exitcode = result.exit_code
 
+    log_helper.end_logging()
     print('')
     print('Flags: A=AutoExec, W=Write, X=Execute')
     print('Exit code: %d - %s' % (exitcode, global_result.name))
