@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/bin/env python
 """
 mraptor_milter
 
@@ -98,18 +98,7 @@ from oletools import olevba, mraptor
 
 from Milter.utils import parse_addr
 
-if sys.version_info[0] <= 2:
-    # Python 2.x
-    if sys.version_info[1] <= 6:
-        # Python 2.6
-        # use is_zipfile backported from Python 2.7:
-        from oletools.thirdparty.zipfile27 import is_zipfile
-    else:
-        # Python 2.7
-        from zipfile import is_zipfile
-else:
-    # Python 3.x+
-    from zipfile import is_zipfile
+from zipfile import is_zipfile
 
 
 
@@ -405,7 +394,7 @@ if __name__ == "__main__":
     daemon.start()
 
     # Using python-daemon - Does not work as-is, need to create the PID file
-    # See https://pypi.python.org/pypi/python-daemon/
+    # See https://pypi.org/project/python-daemon/
     # See PEP-3143: https://www.python.org/dev/peps/pep-3143/
     # import daemon
     # import lockfile
