@@ -41,8 +41,9 @@ file structure and will replace this module some time soon!
 # 2016-09-13       PL: - fixed olefile import for Python 2+3
 #                      - fixed format strings for Python 2.6 (issue #75)
 # 2017-04-23 v0.51 PL: - fixed absolute imports and issue #101
+# 2018-09-11 v0.54 PL: - olefile is now a dependency
 
-__version__ = '0.51'
+__version__ = '0.54'
 
 
 # --- IMPORTS ------------------------------------------------------------------
@@ -66,11 +67,41 @@ _parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
 if not _parent_dir in sys.path:
     sys.path.insert(0, _parent_dir)
 
-from oletools.thirdparty.olefile import olefile
+import olefile
+
+
+# TODO: this is a temporary fix until all logging features are unified in oletools
+def get_logger(name, level=logging.CRITICAL+1):
+    """
+    Create a suitable logger object for this module.
+    The goal is not to change settings of the root logger, to avoid getting
+    other modules' logs on the screen.
+    If a logger exists with same name, reuse it. (Else it would have duplicate
+    handlers and messages would be doubled.)
+    The level is set to CRITICAL+1 by default, to avoid any logging.
+    """
+    # First, test if there is already a logger with the same name, else it
+    # will generate duplicate messages (due to duplicate handlers):
+    if name in logging.Logger.manager.loggerDict:
+        #NOTE: another less intrusive but more "hackish" solution would be to
+        # use getLogger then test if its effective level is not default.
+        logger = logging.getLogger(name)
+        # make sure level is OK:
+        logger.setLevel(level)
+        return logger
+    # get a new logger:
+    logger = logging.getLogger(name)
+    # only add a NullHandler for this logger, it is up to the application
+    # to configure its own logging:
+    logger.addHandler(logging.NullHandler())
+    logger.setLevel(level)
+    return logger
+
+
 
 
 # a global logger object used for debugging:
-log = olefile.get_logger('ppt')
+log = get_logger('ppt')
 
 
 def enable_logging():
@@ -1584,7 +1615,7 @@ def iterative_decompress(stream, size, chunk_size=4096):
 
     decompressor = zlib.decompressobj()
     n_read = 0
-    decomp = ''
+    decomp = b''
     return_err = None
 
     try:
