@@ -5,7 +5,7 @@ Read storages, (sub-)streams, records from xls file
 #
 # === LICENSE ==================================================================
 
-# xls_parser is copyright (c) 2014-2017 Philippe Lagadec (http://www.decalage.info)
+# xls_parser is copyright (c) 2014-2019 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -33,8 +33,10 @@ Read storages, (sub-)streams, records from xls file
 # 2017-11-02 v0.1 CH: - first version
 # 2017-11-02 v0.2 CH: - move some code to record_base.py
 #                        (to avoid copy-and-paste in ppt_parser.py)
+# 2019-01-30 v0.54 PL: - fixed import to avoid mixing installed oletools
+#                        and dev version
 
-__version__ = '0.2'
+__version__ = '0.54'
 
 # -----------------------------------------------------------------------------
 #  TODO:
@@ -56,17 +58,14 @@ import os.path
 from struct import unpack
 import logging
 
-try:
-    from oletools import record_base
-except ImportError:
-    # little hack to allow absolute imports even if oletools is not installed.
-    # Copied from olevba.py
-    PARENT_DIR = os.path.normpath(os.path.dirname(os.path.dirname(
-        os.path.abspath(__file__))))
-    if PARENT_DIR not in sys.path:
-        sys.path.insert(0, PARENT_DIR)
-    del PARENT_DIR
-    from oletools import record_base
+# little hack to allow absolute imports even if oletools is not installed.
+# Copied from olevba.py
+PARENT_DIR = os.path.normpath(os.path.dirname(os.path.dirname(
+    os.path.abspath(__file__))))
+if PARENT_DIR not in sys.path:
+    sys.path.insert(0, PARENT_DIR)
+del PARENT_DIR
+from oletools import record_base
 
 
 # === PYTHON 2+3 SUPPORT ======================================================
@@ -86,21 +85,29 @@ def is_xls(filename):
     returns True if given file is an ole file and contains a Workbook stream
 
     todo: could further check that workbook stream starts with a globals
-    substream
+    substream.
+    See also: oleid.OleID.check_excel
     """
+    xls_file = None
     try:
-        for stream in XlsFile(filename).iter_streams():
+        xls_file = XlsFile(filename)
+        for stream in xls_file.iter_streams():
             if isinstance(stream, WorkbookStream):
                 return True
     except Exception:
-        return False
+        logging.debug('Ignoring exception in is_xls, assume is not xls',
+                      exc_info=True)
+    finally:
+        if xls_file is not None:
+            xls_file.close()
+    return False
 
 
 def read_unicode(data, start_idx, n_chars):
     """ read a unicode string from a XLUnicodeStringNoCch structure """
     # first bit 0x0 --> only low-bytes are saved, all high bytes are 0
     # first bit 0x1 --> 2 bytes per character
-    low_bytes_only = (ord(data[start_idx]) == 0)
+    low_bytes_only = (ord(data[start_idx:start_idx+1]) == 0)
     if low_bytes_only:
         end_idx = start_idx + 1 + n_chars
         return data[start_idx+1:end_idx].decode('ascii'), end_idx
@@ -130,6 +137,8 @@ class XlsFile(record_base.OleRecordFile):
     @classmethod
     def stream_class_for_name(cls, stream_name):
         """ helper for iter_streams """
+        if stream_name == 'Workbook':
+            return WorkbookStream
         return XlsStream
 
 
@@ -346,6 +355,7 @@ class XlsRecordSupBook(XlsRecord):
     LINK_TYPE_EXTERNAL = 'external workbook'
 
     def finish_constructing(self, _):
+        """Finish constructing this record; called at end of constructor."""
         # set defaults
         self.ctab = None
         self.cch = None
