@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/bin/env python
 """
 msodde.py
 
@@ -21,7 +21,7 @@ http://www.decalage.info/python/oletools
 
 # === LICENSE =================================================================
 
-# msodde is copyright (c) 2017-2018 Philippe Lagadec (http://www.decalage.info)
+# msodde is copyright (c) 2017-2019 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -51,27 +51,31 @@ from __future__ import print_function
 
 import argparse
 import os
-from os.path import abspath, dirname
 import sys
-import json
-import logging
 import re
 import csv
 
-# little hack to allow absolute imports even if oletools is not installed
-# Copied from olevba.py
-try:
-    from oletools.thirdparty import olefile
-except ImportError:
-    PARENT_DIR = dirname(dirname(abspath(__file__)))
-    if PARENT_DIR not in sys.path:
-        sys.path.insert(0, PARENT_DIR)
-    del PARENT_DIR
-    from oletools.thirdparty import olefile
+import olefile
+
+# IMPORTANT: it should be possible to run oletools directly as scripts
+# in any directory without installing them with pip or setup.py.
+# In that case, relative imports are NOT usable.
+# And to enable Python 2+3 compatibility, we need to use absolute imports,
+# so we add the oletools parent folder to sys.path (absolute+normalized path):
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if _parent_dir not in sys.path:
+    sys.path.insert(0, _parent_dir)
 
 from oletools import ooxml
 from oletools import xls_parser
 from oletools import rtfobj
+from oletools.ppt_record_parser import is_ppt
+from oletools import crypto
+from oletools.common.io_encoding import ensure_stdout_handles_unicode
+from oletools.common.log_helper import log_helper
 
 # -----------------------------------------------------------------------------
 # CHANGELOG:
@@ -93,8 +97,13 @@ from oletools import rtfobj
 # 2018-01-11       PL: - fixed issue #242 (apply unquote to fldSimple tags)
 # 2018-01-10       CH: - add single-xml files (Word 2003/2007+ / Excel 2003)
 # 2018-03-21       CH: - added detection for various CSV formulas (issue #259)
+# 2018-09-11 v0.54 PL: - olefile is now a dependency
+# 2018-10-25       CH: - detect encryption and raise error if detected
+# 2019-03-25       CH: - added decryption of password-protected files
+# 2019-07-17 v0.55 CH: - fixed issue #267, unicode error on Python 2
 
-__version__ = '0.53'
+
+__version__ = '0.55'
 
 # -----------------------------------------------------------------------------
 # TODO: field codes can be in headers/footers/comments - parse these
@@ -220,114 +229,12 @@ THIS IS WORK IN PROGRESS - Check updates regularly!
 Please report any issue at https://github.com/decalage2/oletools/issues
 """ % __version__
 
-BANNER_JSON = dict(type='meta', version=__version__, name='msodde',
-                   link='http://decalage.info/python/oletools',
-                   message='THIS IS WORK IN PROGRESS - Check updates regularly! '
-                           'Please report any issue at '
-                           'https://github.com/decalage2/oletools/issues')
-
 # === LOGGING =================================================================
 
 DEFAULT_LOG_LEVEL = "warning"  # Default log level
-LOG_LEVELS = {
-    'debug': logging.DEBUG,
-    'info': logging.INFO,
-    'warning': logging.WARNING,
-    'error': logging.ERROR,
-    'critical': logging.CRITICAL
-}
-
-
-class NullHandler(logging.Handler):
-    """
-    Log Handler without output, to avoid printing messages if logging is not
-    configured by the main application.
-    Python 2.7 has logging.NullHandler, but this is necessary for 2.6:
-    see https://docs.python.org/2.6/library/logging.html#configuring-logging-for-a-library
-    """
-    def emit(self, record):
-        pass
-
-
-def get_logger(name, level=logging.CRITICAL+1):
-    """
-    Create a suitable logger object for this module.
-    The goal is not to change settings of the root logger, to avoid getting
-    other modules' logs on the screen.
-    If a logger exists with same name, reuse it. (Else it would have duplicate
-    handlers and messages would be doubled.)
-    The level is set to CRITICAL+1 by default, to avoid any logging.
-    """
-    # First, test if there is already a logger with the same name, else it
-    # will generate duplicate messages (due to duplicate handlers):
-    if name in logging.Logger.manager.loggerDict:
-        # NOTE: another less intrusive but more "hackish" solution would be to
-        # use getLogger then test if its effective level is not default.
-        logger = logging.getLogger(name)
-        # make sure level is OK:
-        logger.setLevel(level)
-        return logger
-    # get a new logger:
-    logger = logging.getLogger(name)
-    # only add a NullHandler for this logger, it is up to the application
-    # to configure its own logging:
-    logger.addHandler(NullHandler())
-    logger.setLevel(level)
-    return logger
 
 # a global logger object used for debugging:
-log = get_logger('msodde')
-
-
-# === UNICODE IN PY2 =========================================================
-
-def ensure_stdout_handles_unicode():
-    """ Ensure stdout can handle unicode by wrapping it if necessary
-
-    Required e.g. if output of this script is piped or redirected in a linux
-    shell, since then sys.stdout.encoding is ascii and cannot handle
-    print(unicode). In that case we need to find some compatible encoding and
-    wrap sys.stdout into a encoder following (many thanks!)
-    https://stackoverflow.com/a/1819009 or https://stackoverflow.com/a/20447935
-
-    Can be undone by setting sys.stdout = sys.__stdout__
-    """
-    import codecs
-    import locale
-
-    # do not re-wrap
-    if isinstance(sys.stdout, codecs.StreamWriter):
-        return
-
-    # try to find encoding for sys.stdout
-    encoding = None
-    try:
-        encoding = sys.stdout.encoding
-    except AttributeError:              # variable "encoding" might not exist
-        pass
-
-    if encoding not in (None, '', 'ascii'):
-        return   # no need to wrap
-
-    # try to find an encoding that can handle unicode
-    try:
-        encoding = locale.getpreferredencoding()
-    except Exception:
-        pass
-
-    # fallback if still no encoding available
-    if encoding in (None, '', 'ascii'):
-        encoding = 'utf8'
-
-    # logging is probably not initialized yet, but just in case
-    log.debug('wrapping sys.stdout with encoder using {0}'.format(encoding))
-
-    wrapper = codecs.getwriter(encoding)
-    sys.stdout = wrapper(sys.stdout)
-
-
-if sys.version_info.major < 3:
-    ensure_stdout_handles_unicode()   # e.g. for print(text) in main()
+logger = log_helper.get_or_create_silent_logger('msodde')
 
 
 # === ARGUMENT PARSING =======================================================
@@ -361,6 +268,9 @@ def process_args(cmd_line_args=None):
                         default=DEFAULT_LOG_LEVEL,
                         help="logging level debug/info/warning/error/critical "
                              "(default=%(default)s)")
+    parser.add_argument("-p", "--password", type=str, action='append',
+                        help='if encrypted office files are encountered, try '
+                             'decryption with this password. May be repeated.')
     filter_group = parser.add_argument_group(
         title='Filter which OpenXML field commands are returned',
         description='Only applies to OpenXML (e.g. docx) and rtf, not to OLE '
@@ -404,14 +314,13 @@ def process_doc_field(data):
     """ check if field instructions start with DDE
 
     expects unicode input, returns unicode output (empty if not dde) """
-    log.debug('processing field \'{0}\''.format(data))
+    logger.debug(u'processing field \'{0}\''.format(data))
 
     if data.lstrip().lower().startswith(u'dde'):
         return data
-    elif data.lstrip().lower().startswith(u'\x00d\x00d\x00e\x00'):
+    if data.lstrip().lower().startswith(u'\x00d\x00d\x00e\x00'):
         return data
-    else:
-        return u''
+    return u''
 
 
 OLE_FIELD_START = 0x13
@@ -435,14 +344,14 @@ def process_doc_stream(stream):
     while True:
         idx += 1
         char = stream.read(1)    # loop over every single byte
-        if len(char) == 0:
+        if len(char) == 0:                   # pylint: disable=len-as-condition
             break
         else:
             char = ord(char)
 
         if char == OLE_FIELD_START:
             if have_start and max_size_exceeded:
-                log.debug('big field was not a field after all')
+                logger.debug('big field was not a field after all')
             have_start = True
             have_sep = False
             max_size_exceeded = False
@@ -454,7 +363,7 @@ def process_doc_stream(stream):
         # now we are after start char but not at end yet
         if char == OLE_FIELD_SEP:
             if have_sep:
-                log.debug('unexpected field: has multiple separators!')
+                logger.debug('unexpected field: has multiple separators!')
             have_sep = True
         elif char == OLE_FIELD_END:
             # have complete field now, process it
@@ -472,8 +381,8 @@ def process_doc_stream(stream):
             if max_size_exceeded:
                 pass
             elif len(field_contents) > OLE_FIELD_MAX_SIZE:
-                log.debug('field exceeds max size of {0}. Ignore rest'
-                          .format(OLE_FIELD_MAX_SIZE))
+                logger.debug('field exceeds max size of {0}. Ignore rest'
+                             .format(OLE_FIELD_MAX_SIZE))
                 max_size_exceeded = True
 
             # appending a raw byte to a unicode string here. Not clean but
@@ -490,25 +399,26 @@ def process_doc_stream(stream):
                 field_contents += u'?'
 
     if max_size_exceeded:
-        log.debug('big field was not a field after all')
+        logger.debug('big field was not a field after all')
 
-    log.debug('Checked {0} characters, found {1} fields'
-              .format(idx, len(result_parts)))
+    logger.debug('Checked {0} characters, found {1} fields'
+                 .format(idx, len(result_parts)))
 
     return result_parts
 
 
-def process_doc(filepath):
+def process_doc(ole):
     """
     find dde links in word ole (.doc/.dot) file
 
+    Checks whether files is ppt and returns empty immediately in that case
+    (ppt files cannot contain DDE-links to my knowledge)
+
     like process_xml, returns a concatenated unicode string of dde links or
     empty if none were found. dde-links will still begin with the dde[auto] key
     word (possibly after some whitespace)
     """
-    log.debug('process_doc')
-    ole = olefile.OleFileIO(filepath, path_encoding=None)
-
+    logger.debug('process_doc')
     links = []
     for sid, direntry in enumerate(ole.direntries):
         is_orphan = direntry is None
@@ -516,12 +426,11 @@ def process_doc(filepath):
             # this direntry is not part of the tree --> unused or orphan
             direntry = ole._load_direntry(sid)
         is_stream = direntry.entry_type == olefile.STGTY_STREAM
-        log.debug('direntry {:2d} {}: {}'
-                  .format(sid, '[orphan]' if is_orphan else direntry.name,
-                          'is stream of size {}'.format(direntry.size)
-                          if is_stream else
-                          'no stream ({})'
-                          .format(direntry.entry_type)))
+        logger.debug('direntry {:2d} {}: {}'
+                     .format(sid, '[orphan]' if is_orphan else direntry.name,
+                             'is stream of size {}'.format(direntry.size)
+                             if is_stream else
+                             'no stream ({})'.format(direntry.entry_type)))
         if is_stream:
             new_parts = process_doc_stream(
                 ole._open(direntry.isectStart, direntry.size))
@@ -535,17 +444,23 @@ def process_xls(filepath):
     """ find dde links in excel ole file """
 
     result = []
-    for stream in xls_parser.XlsFile(filepath).iter_streams():
-        if not isinstance(stream, xls_parser.WorkbookStream):
-            continue
-        for record in stream.iter_records():
-            if not isinstance(record, xls_parser.XlsRecordSupBook):
+    xls_file = None
+    try:
+        xls_file = xls_parser.XlsFile(filepath)
+        for stream in xls_file.iter_streams():
+            if not isinstance(stream, xls_parser.WorkbookStream):
                 continue
-            if record.support_link_type in (
-                    xls_parser.XlsRecordSupBook.LINK_TYPE_OLE_DDE,
-                    xls_parser.XlsRecordSupBook.LINK_TYPE_EXTERNAL):
-                result.append(record.virt_path.replace(u'\u0003', u' '))
-    return u'\n'.join(result)
+            for record in stream.iter_records():
+                if not isinstance(record, xls_parser.XlsRecordSupBook):
+                    continue
+                if record.support_link_type in (
+                        xls_parser.XlsRecordSupBook.LINK_TYPE_OLE_DDE,
+                        xls_parser.XlsRecordSupBook.LINK_TYPE_EXTERNAL):
+                    result.append(record.virt_path.replace(u'\u0003', u' '))
+        return u'\n'.join(result)
+    finally:
+        if xls_file is not None:
+            xls_file.close()
 
 
 def process_docx(filepath, field_filter_mode=None):
@@ -580,7 +495,8 @@ def process_docx(filepath, field_filter_mode=None):
             else:
                 elem = curr_elem
             if elem is None:
-                raise BadOOXML(filepath, 'Got "None"-Element from iter_xml')
+                raise ooxml.BadOOXML(filepath,
+                                     'Got "None"-Element from iter_xml')
 
             # check if FLDCHARTYPE and whether "begin" or "end" tag
             attrib_type = elem.attrib.get(ATTR_W_FLDCHARTYPE[0]) or \
@@ -590,7 +506,7 @@ def process_docx(filepath, field_filter_mode=None):
                     level += 1
                 if attrib_type == "end":
                     level -= 1
-                    if level == 0 or level == -1:  # edge-case; level gets -1
+                    if level in (0, -1):  # edge-case; level gets -1
                         all_fields.append(ddetext)
                         ddetext = u''
                         level = 0  # reset edge-case
@@ -601,7 +517,7 @@ def process_docx(filepath, field_filter_mode=None):
                 ddetext += unquote(elem.text)
 
     # apply field command filter
-    log.debug('filtering with mode "{0}"'.format(field_filter_mode))
+    logger.debug('filtering with mode "{0}"'.format(field_filter_mode))
     if field_filter_mode in (FIELD_FILTER_ALL, None):
         clean_fields = all_fields
     elif field_filter_mode == FIELD_FILTER_DDE:
@@ -619,6 +535,7 @@ def process_docx(filepath, field_filter_mode=None):
 
 
 def unquote(field):
+    """TODO: document what exactly is happening here..."""
     if "QUOTE" not in field or NO_QUOTES:
         return field
     # split into components
@@ -660,8 +577,8 @@ def field_is_blacklisted(contents):
         index = FIELD_BLACKLIST_CMDS.index(words[0].lower())
     except ValueError:    # first word is no blacklisted command
         return False
-    log.debug('trying to match "{0}" to blacklist command {1}'
-              .format(contents, FIELD_BLACKLIST[index]))
+    logger.debug(u'trying to match "{0}" to blacklist command {1}'
+                 .format(contents, FIELD_BLACKLIST[index]))
     _, nargs_required, nargs_optional, sw_with_arg, sw_solo, sw_format \
         = FIELD_BLACKLIST[index]
 
@@ -672,12 +589,13 @@ def field_is_blacklisted(contents):
             break
         nargs += 1
     if nargs < nargs_required:
-        log.debug('too few args: found {0}, but need at least {1} in "{2}"'
-                  .format(nargs, nargs_required, contents))
+        logger.debug(u'too few args: found {0}, but need at least {1} in "{2}"'
+                     .format(nargs, nargs_required, contents))
         return False
-    elif nargs > nargs_required + nargs_optional:
-        log.debug('too many args: found {0}, but need at most {1}+{2} in "{3}"'
-                  .format(nargs, nargs_required, nargs_optional, contents))
+    if nargs > nargs_required + nargs_optional:
+        logger.debug(u'too many args: found {0}, but need at most {1}+{2} in '
+                     u'"{3}"'
+                     .format(nargs, nargs_required, nargs_optional, contents))
         return False
 
     # check switches
@@ -686,15 +604,15 @@ def field_is_blacklisted(contents):
     for word in words[1+nargs:]:
         if expect_arg:            # this is an argument for the last switch
             if arg_choices and (word not in arg_choices):
-                log.debug('Found invalid switch argument "{0}" in "{1}"'
-                          .format(word, contents))
+                logger.debug(u'Found invalid switch argument "{0}" in "{1}"'
+                             .format(word, contents))
                 return False
             expect_arg = False
             arg_choices = []   # in general, do not enforce choices
             continue           # "no further questions, your honor"
         elif not FIELD_SWITCH_REGEX.match(word):
-            log.debug('expected switch, found "{0}" in "{1}"'
-                      .format(word, contents))
+            logger.debug(u'expected switch, found "{0}" in "{1}"'
+                         .format(word, contents))
             return False
         # we want a switch and we got a valid one
         switch = word[1]
@@ -715,8 +633,8 @@ def field_is_blacklisted(contents):
             if 'numeric' in sw_format:
                 arg_choices = []  # too many choices to list them here
         else:
-            log.debug('unexpected switch {0} in "{1}"'
-                      .format(switch, contents))
+            logger.debug(u'unexpected switch {0} in "{1}"'
+                         .format(switch, contents))
             return False
 
     # if nothing went wrong sofar, the contents seems to match the blacklist
@@ -731,7 +649,7 @@ def process_xlsx(filepath):
         tag = elem.tag.lower()
         if tag == 'ddelink' or tag.endswith('}ddelink'):
             # we have found a dde link. Try to get more info about it
-            link_info = ['DDE-Link']
+            link_info = []
             if 'ddeService' in elem.attrib:
                 link_info.append(elem.attrib['ddeService'])
             if 'ddeTopic' in elem.attrib:
@@ -741,30 +659,29 @@ def process_xlsx(filepath):
     # binary parts, e.g. contained in .xlsb
     for subfile, content_type, handle in parser.iter_non_xml():
         try:
-            logging.info('Parsing non-xml subfile {0} with content type {1}'
-                         .format(subfile, content_type))
+            logger.info('Parsing non-xml subfile {0} with content type {1}'
+                        .format(subfile, content_type))
             for record in xls_parser.parse_xlsb_part(handle, content_type,
                                                      subfile):
-                logging.debug('{0}: {1}'.format(subfile, record))
+                logger.debug('{0}: {1}'.format(subfile, record))
                 if isinstance(record, xls_parser.XlsbBeginSupBook) and \
                         record.link_type == \
                         xls_parser.XlsbBeginSupBook.LINK_TYPE_DDE:
-                    dde_links.append('DDE-Link ' + record.string1 + ' ' +
-                                     record.string2)
-        except Exception:
+                    dde_links.append(record.string1 + ' ' + record.string2)
+        except Exception as exc:
             if content_type.startswith('application/vnd.ms-excel.') or \
                content_type.startswith('application/vnd.ms-office.'):  # pylint: disable=bad-indentation
                 # should really be able to parse these either as xml or records
-                log_func = logging.warning
+                log_func = logger.warning
             elif content_type.startswith('image/') or content_type == \
                     'application/vnd.openxmlformats-officedocument.' + \
                     'spreadsheetml.printerSettings':
                 # understandable that these are not record-base
-                log_func = logging.debug
+                log_func = logger.debug
             else:   # default
-                log_func = logging.info
-            log_func('Failed to parse {0} of content type {1}'
-                     .format(subfile, content_type))
+                log_func = logger.info
+            log_func('Failed to parse {0} of content type {1} ("{2}")'
+                     .format(subfile, content_type, str(exc)))
             # in any case: continue with next
 
     return u'\n'.join(dde_links)
@@ -782,15 +699,16 @@ class RtfFieldParser(rtfobj.RtfParser):
 
     def open_destination(self, destination):
         if destination.cword == b'fldinst':
-            log.debug('*** Start field data at index %Xh' % destination.start)
+            logger.debug('*** Start field data at index %Xh'
+                         % destination.start)
 
     def close_destination(self, destination):
         if destination.cword == b'fldinst':
-            log.debug('*** Close field data at index %Xh' % self.index)
-            log.debug('Field text: %r' % destination.data)
+            logger.debug('*** Close field data at index %Xh' % self.index)
+            logger.debug('Field text: %r' % destination.data)
             # remove extra spaces and newline chars:
             field_clean = destination.data.translate(None, b'\r\n').strip()
-            log.debug('Cleaned Field text: %r' % field_clean)
+            logger.debug('Cleaned Field text: %r' % field_clean)
             self.fields.append(field_clean)
 
     def control_symbol(self, matchobject):
@@ -812,8 +730,8 @@ def process_rtf(file_handle, field_filter_mode=None):
     rtfparser.parse()
     all_fields = [field.decode('ascii') for field in rtfparser.fields]
     # apply field command filter
-    log.debug('found {1} fields, filtering with mode "{0}"'
-              .format(field_filter_mode, len(all_fields)))
+    logger.debug('found {1} fields, filtering with mode "{0}"'
+                 .format(field_filter_mode, len(all_fields)))
     if field_filter_mode in (FIELD_FILTER_ALL, None):
         clean_fields = all_fields
     elif field_filter_mode == FIELD_FILTER_DDE:
@@ -852,16 +770,22 @@ def process_csv(filepath):
     chars the same way that excel does. Tested to some extend in unittests.
 
     This can only find DDE-links, no other "suspicious" constructs (yet).
-    """
 
+    Cannot deal with unicode files yet (need more than just use uopen()).
+    """
     results = []
-    with open(filepath, 'r') as file_handle:
+    if sys.version_info.major <= 2:
+        open_arg = dict(mode='rb')
+    else:
+        open_arg = dict(newline='')
+    with open(filepath, **open_arg) as file_handle:
+        # TODO: here we should not assume this is a file on disk, filepath can be a file object
         results, dialect = process_csv_dialect(file_handle, CSV_DELIMITERS)
         is_small = file_handle.tell() < CSV_SMALL_THRESH
 
         if is_small and not results:
             # easy to mis-sniff small files. Try different delimiters
-            log.debug('small file, no results; try all delimiters')
+            logger.debug('small file, no results; try all delimiters')
             file_handle.seek(0)
             other_delim = CSV_DELIMITERS.replace(dialect.delimiter, '')
             for delim in other_delim:
@@ -869,12 +793,13 @@ def process_csv(filepath):
                     file_handle.seek(0)
                     results, _ = process_csv_dialect(file_handle, delim)
                 except csv.Error:   # e.g. sniffing fails
-                    log.debug('failed to csv-parse with delimiter {0!r}'
-                              .format(delim))
+                    logger.debug('failed to csv-parse with delimiter {0!r}'
+                                 .format(delim))
 
         if is_small and not results:
             # try whole file as single cell, since sniffing fails in this case
-            log.debug('last attempt: take whole file as single unquoted cell')
+            logger.debug('last attempt: take whole file as single unquoted '
+                         'cell')
             file_handle.seek(0)
             match = CSV_DDE_FORMAT.match(file_handle.read(CSV_SMALL_THRESH))
             if match:
@@ -885,14 +810,13 @@ def process_csv(filepath):
 
 def process_csv_dialect(file_handle, delimiters):
     """ helper for process_csv: process with a specific csv dialect """
-
     # determine dialect = delimiter chars, quote chars, ...
     dialect = csv.Sniffer().sniff(file_handle.read(CSV_SMALL_THRESH),
                                   delimiters=delimiters)
     dialect.strict = False     # microsoft is never strict
-    log.debug('sniffed csv dialect with delimiter {0!r} '
-              'and quote char {1!r}'
-              .format(dialect.delimiter, dialect.quotechar))
+    logger.debug('sniffed csv dialect with delimiter {0!r} '
+                 'and quote char {1!r}'
+                 .format(dialect.delimiter, dialect.quotechar))
 
     # rewind file handle to start
     file_handle.seek(0)
@@ -932,7 +856,7 @@ def process_excel_xml(filepath):
                 break
         if formula is None:
             continue
-        log.debug('found cell with formula {0}'.format(formula))
+        logger.debug(u'found cell with formula {0}'.format(formula))
         match = re.match(XML_DDE_FORMAT, formula)
         if match:
             dde_links.append(u' '.join(match.groups()[:2]))
@@ -942,45 +866,105 @@ def process_excel_xml(filepath):
 def process_file(filepath, field_filter_mode=None):
     """ decides which of the process_* functions to call """
     if olefile.isOleFile(filepath):
-        log.debug('Is OLE. Checking streams to see whether this is xls')
+        logger.debug('Is OLE. Checking streams to see whether this is xls')
         if xls_parser.is_xls(filepath):
-            log.debug('Process file as excel 2003 (xls)')
+            logger.debug('Process file as excel 2003 (xls)')
             return process_xls(filepath)
-        else:
-            log.debug('Process file as word 2003 (doc)')
-            return process_doc(filepath)
+        if is_ppt(filepath):
+            logger.debug('is ppt - cannot have DDE')
+            return u''
+        logger.debug('Process file as word 2003 (doc)')
+        with olefile.OleFileIO(filepath, path_encoding=None) as ole:
+            return process_doc(ole)
 
     with open(filepath, 'rb') as file_handle:
+        # TODO: here we should not assume this is a file on disk, filepath can be a file object
         if file_handle.read(4) == RTF_START:
-            log.debug('Process file as rtf')
+            logger.debug('Process file as rtf')
             return process_rtf(file_handle, field_filter_mode)
 
     try:
         doctype = ooxml.get_type(filepath)
-        log.debug('Detected file type: {0}'.format(doctype))
+        logger.debug('Detected file type: {0}'.format(doctype))
     except Exception as exc:
-        log.debug('Exception trying to xml-parse file: {0}'.format(exc))
+        logger.debug('Exception trying to xml-parse file: {0}'.format(exc))
         doctype = None
 
     if doctype == ooxml.DOCTYPE_EXCEL:
-        log.debug('Process file as excel 2007+ (xlsx)')
+        logger.debug('Process file as excel 2007+ (xlsx)')
         return process_xlsx(filepath)
-    elif doctype in (ooxml.DOCTYPE_EXCEL_XML, ooxml.DOCTYPE_EXCEL_XML2003):
-        log.debug('Process file as xml from excel 2003/2007+')
+    if doctype in (ooxml.DOCTYPE_EXCEL_XML, ooxml.DOCTYPE_EXCEL_XML2003):
+        logger.debug('Process file as xml from excel 2003/2007+')
         return process_excel_xml(filepath)
-    elif doctype in (ooxml.DOCTYPE_WORD_XML, ooxml.DOCTYPE_WORD_XML2003):
-        log.debug('Process file as xml from word 2003/2007+')
+    if doctype in (ooxml.DOCTYPE_WORD_XML, ooxml.DOCTYPE_WORD_XML2003):
+        logger.debug('Process file as xml from word 2003/2007+')
         return process_docx(filepath)
-    elif doctype is None:
-        log.debug('Process file as csv')
+    if doctype is None:
+        logger.debug('Process file as csv')
         return process_csv(filepath)
-    else:  # could be docx; if not: this is the old default code path
-        log.debug('Process file as word 2007+ (docx)')
-        return process_docx(filepath, field_filter_mode)
+    # could be docx; if not: this is the old default code path
+    logger.debug('Process file as word 2007+ (docx)')
+    return process_docx(filepath, field_filter_mode)
 
 
 # === MAIN =================================================================
 
+
+def process_maybe_encrypted(filepath, passwords=None, crypto_nesting=0,
+                            **kwargs):
+    """
+    Process a file that might be encrypted.
+
+    Calls :py:func:`process_file` and if that fails tries to decrypt and
+    process the result. Based on recommendation in module doc string of
+    :py:mod:`oletools.crypto`.
+
+    :param str filepath: path to file on disc.
+    :param passwords: list of passwords (str) to try for decryption or None
+    :param int crypto_nesting: How many decryption layers were already used to
+                               get the given file.
+    :param kwargs: same as :py:func:`process_file`
+    :returns: same as :py:func:`process_file`
+    """
+    # TODO: here filepath may also be a file in memory, it's not necessarily on disk
+    result = u''
+    try:
+        result = process_file(filepath, **kwargs)
+        if not crypto.is_encrypted(filepath):
+            return result
+    except Exception:
+        logger.debug('Ignoring exception:', exc_info=True)
+        if not crypto.is_encrypted(filepath):
+            raise
+
+    # we reach this point only if file is encrypted
+    # check if this is an encrypted file in an encrypted file in an ...
+    if crypto_nesting >= crypto.MAX_NESTING_DEPTH:
+        raise crypto.MaxCryptoNestingReached(crypto_nesting, filepath)
+
+    decrypted_file = None
+    if passwords is None:
+        passwords = crypto.DEFAULT_PASSWORDS
+    else:
+        passwords = list(passwords) + crypto.DEFAULT_PASSWORDS
+    try:
+        logger.debug('Trying to decrypt file')
+        decrypted_file = crypto.decrypt(filepath, passwords)
+        if not decrypted_file:
+            logger.error('Decrypt failed, run with debug output to get details')
+            raise crypto.WrongEncryptionPassword(filepath)
+        logger.info('Analyze decrypted file')
+        result = process_maybe_encrypted(decrypted_file, passwords,
+                                         crypto_nesting+1, **kwargs)
+    finally:     # clean up
+        try:     # (maybe file was not yet created)
+            os.unlink(decrypted_file)
+        except Exception:
+            logger.debug('Ignoring exception closing decrypted file:',
+                         exc_info=True)
+    return result
+
+
 def main(cmd_line_args=None):
     """ Main function, called if this file is called as a script
 
@@ -993,50 +977,30 @@ def main(cmd_line_args=None):
     # Setup logging to the console:
     # here we use stdout instead of stderr by default, so that the output
     # can be redirected properly.
-    logging.basicConfig(level=LOG_LEVELS[args.loglevel], stream=sys.stdout,
-                        format='%(levelname)-8s %(message)s')
-    # enable logging in the modules:
-    log.setLevel(logging.NOTSET)
-
-    if args.json and args.loglevel.lower() == 'debug':
-        log.warning('Debug log output will not be json-compatible!')
+    log_helper.enable_logging(args.json, args.loglevel, stream=sys.stdout)
 
     if args.nounquote:
         global NO_QUOTES
         NO_QUOTES = True
 
-    if args.json:
-        jout = []
-        jout.append(BANNER_JSON)
-    else:
-        # print banner with version
-        print(BANNER)
-
-    if not args.json:
-        print('Opening file: %s' % args.filepath)
+    logger.print_str(BANNER)
+    logger.print_str('Opening file: %s' % args.filepath)
 
     text = ''
     return_code = 1
     try:
-        text = process_file(args.filepath, args.field_filter_mode)
+        text = process_maybe_encrypted(
+            args.filepath, args.password,
+            field_filter_mode=args.field_filter_mode)
         return_code = 0
     except Exception as exc:
-        if args.json:
-            jout.append(dict(type='error', error=type(exc).__name__,
-                             message=str(exc)))
-        else:
-            raise  # re-raise last known exception, keeping trace intact
+        logger.exception(str(exc))
 
-    if args.json:
-        for line in text.splitlines():
-            if line.strip():
-                jout.append(dict(type='dde-link', link=line.strip()))
-        json.dump(jout, sys.stdout, check_circular=False, indent=4)
-        print()   # add a newline after closing "]"
-        return return_code  # required if we catch an exception in json-mode
-    else:
-        print ('DDE Links:')
-        print(text)
+    logger.print_str('DDE Links:')
+    for link in text.splitlines():
+        logger.print_str(text, type='dde-link')
+
+    log_helper.end_logging()
 
     return return_code
 
