@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/bin/env python
 """
 oleobj.py
 
@@ -14,7 +14,7 @@ http://www.decalage.info/python/oletools
 
 # === LICENSE =================================================================
 
-# oleobj is copyright (c) 2015-2018 Philippe Lagadec (http://www.decalage.info)
+# oleobj is copyright (c) 2015-2022 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -50,27 +50,28 @@ import os
 import re
 import sys
 import io
-from zipfile import is_zipfile, ZipFile
+from zipfile import is_zipfile
+import random
+
+import olefile
 
 # IMPORTANT: it should be possible to run oletools directly as scripts
 # in any directory without installing them with pip or setup.py.
 # In that case, relative imports are NOT usable.
 # And to enable Python 2+3 compatibility, we need to use absolute imports,
 # so we add the oletools parent folder to sys.path (absolute+normalized path):
-try:
-    from oletools.thirdparty import olefile
-except ImportError:
-    import os.path
-    PARENT_DIR = os.path.normpath(os.path.dirname(os.path.dirname(
-        os.path.abspath(__file__))))
-    if PARENT_DIR not in sys.path:
-        sys.path.insert(0, PARENT_DIR)
-    del PARENT_DIR
-    from oletools.thirdparty import olefile
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if _parent_dir not in sys.path:
+    sys.path.insert(0, _parent_dir)
+
 from oletools.thirdparty import xglob
 from oletools.ppt_record_parser import (is_ppt, PptFile,
                                         PptRecordExOleVbaActiveXAtom)
-from oletools.ooxml import ZipSubFile
+from oletools.ooxml import XmlParser
+from oletools.common.io_encoding import ensure_stdout_handles_unicode
 
 # -----------------------------------------------------------------------------
 # CHANGELOG:
@@ -84,8 +85,12 @@ from oletools.ooxml import ZipSubFile
 # 2018-01-18 v0.52 CH: - added support for zipped-xml-based types (docx, pptx,
 #                        xlsx), and ppt
 # 2018-03-27       PL: - fixed issue #274 in read_length_prefixed_string
+# 2018-09-11 v0.54 PL: - olefile is now a dependency
+# 2018-10-30       SA: - added detection of external links (PR #317)
+# 2020-03-03 v0.56 PL: - fixed bug #541, "Ole10Native" is case-insensitive
+# 2022-01-28 v0.60 PL: - added detection of customUI tags
 
-__version__ = '0.52.4'
+__version__ = '0.60.1'
 
 # -----------------------------------------------------------------------------
 # TODO:
@@ -178,6 +183,10 @@ else:
     NULL_CHAR = 0     # pylint: disable=redefined-variable-type
     xrange = range    # pylint: disable=redefined-builtin, invalid-name
 
+OOXML_RELATIONSHIP_TAG = '{http://schemas.openxmlformats.org/package/2006/relationships}Relationship'
+# There are several customUI tags for different versions of Office:
+TAG_CUSTOMUI_2007 = "{http://schemas.microsoft.com/office/2006/01/customui}customUI"
+TAG_CUSTOMUI_2010 = "{http://schemas.microsoft.com/office/2009/07/customui}customUI"
 
 # === GLOBAL VARIABLES ========================================================
 
@@ -203,6 +212,30 @@ RETURN_ERR_ARGS = 2    # reserve for OptionParser.parse_args
 RETURN_ERR_STREAM = 4  # error opening/parsing a stream
 RETURN_ERR_DUMP = 8    # error dumping data from stream to file
 
+# Not sure if they can all be "External", but just in case
+BLACKLISTED_RELATIONSHIP_TYPES = [
+    'attachedTemplate',
+    'externalLink',
+    'externalLinkPath',
+    'externalReference',
+    'frame',
+    'hyperlink',
+    'officeDocument',
+    'oleObject',
+    'package',
+    'slideUpdateUrl',
+    'slideMaster',
+    'slide',
+    'slideUpdateInfo',
+    'subDocument',
+    'worksheet'
+]
+
+# Save maximum length of a filename
+MAX_FILENAME_LENGTH = 255
+
+# Max attempts at generating a non-existent random file name
+MAX_FILENAME_ATTEMPTS = 100
 
 # === FUNCTIONS ===============================================================
 
@@ -468,11 +501,40 @@ class OleObject(object):
             self.extra_data = data[index+self.data_size:]
 
 
-def sanitize_filename(filename, replacement='_', max_length=200):
-    """compute basename of filename. Replaces all non-whitelisted characters.
-       The returned filename is always a ascii basename of the file."""
+def shorten_filename(fname, max_len):
+    """Create filename shorter than max_len, trying to preserve suffix."""
+    # simple cases:
+    if not max_len:
+        return fname
+    name_len = len(fname)
+    if name_len < max_len:
+        return fname
+
+    idx = fname.rfind('.')
+    if idx == -1:
+        return fname[:max_len]
+
+    suffix_len = name_len - idx  # length of suffix including '.'
+    if suffix_len > max_len:
+        return fname[:max_len]
+
+    # great, can preserve suffix
+    return fname[:max_len-suffix_len] + fname[idx:]
+
+
+def sanitize_filename(filename, replacement='_',
+                      max_len=MAX_FILENAME_LENGTH):
+    """
+    Return filename that is save to work with.
+
+    Removes path components, replaces all non-whitelisted characters (so output
+    is always a pure-ascii string), replaces '..' and '  ' and shortens to
+    given max length, trying to preserve suffix.
+
+    Might return empty string
+    """
     basepath = os.path.basename(filename).strip()
-    sane_fname = re.sub(u'[^a-zA-Z0-9.\\-_ ]', replacement, basepath)
+    sane_fname = re.sub(u'[^a-zA-Z0-9.\-_ ]', replacement, basepath)
     sane_fname = str(sane_fname)    # py3: does nothing;   py2: unicode --> str
 
     while ".." in sane_fname:
@@ -481,14 +543,71 @@ def sanitize_filename(filename, replacement='_', max_length=200):
     while "  " in sane_fname:
         sane_fname = sane_fname.replace('  ', ' ')
 
-    if not filename:
-        sane_fname = 'NONAME'
+    # limit filename length, try to preserve suffix
+    return shorten_filename(sane_fname, max_len)
 
-    # limit filename length
-    if max_length:
-        sane_fname = sane_fname[:max_length]
 
-    return sane_fname
+def get_sane_embedded_filenames(filename, src_path, tmp_path, max_len,
+                                noname_index):
+    """
+    Get some sane filenames out of path information, preserving file suffix.
+
+    Returns several canddiates, first with suffix, then without, then random
+    with suffix and finally one last attempt ignoring max_len using arg
+    `noname_index`.
+
+    In some malware examples, filename (on which we relied sofar exclusively
+    for this) is empty or " ", but src_path and tmp_path contain paths with
+    proper file names. Try to extract filename from any of those.
+
+    Preservation of suffix is especially important since that controls how
+    windoze treats the file.
+    """
+    suffixes = []
+    candidates_without_suffix = []  # remember these as fallback
+    for candidate in (filename, src_path, tmp_path):
+        # remove path component. Could be from linux, mac or windows
+        idx = max(candidate.rfind('/'), candidate.rfind('\\'))
+        candidate = candidate[idx+1:].strip()
+
+        # sanitize
+        candidate = sanitize_filename(candidate, max_len=max_len)
+
+        if not candidate:
+            continue    # skip whitespace-only
+
+        # identify suffix. Dangerous suffixes are all short
+        idx = candidate.rfind('.')
+        if idx is -1:
+            candidates_without_suffix.append(candidate)
+            continue
+        elif idx < len(candidate)-5:
+            candidates_without_suffix.append(candidate)
+            continue
+
+        # remember suffix
+        suffixes.append(candidate[idx:])
+
+        yield candidate
+
+    # parts with suffix not good enough? try those without one
+    for candidate in candidates_without_suffix:
+        yield candidate
+
+    # then try random
+    suffixes.append('')  # ensure there is something in there
+    for _ in range(MAX_FILENAME_ATTEMPTS):
+        for suffix in suffixes:
+            leftover_len = max_len - len(suffix)
+            if leftover_len < 1:
+                continue
+            name = ''.join(random.sample('abcdefghijklmnopqrstuvwxyz',
+                                         min(26, leftover_len)))
+            yield name + suffix
+
+    # still not returned? Then we have to make up a name ourselves
+    # do not care any more about max_len (maybe it was 0 or negative)
+    yield 'oleobj_%03d' % noname_index
 
 
 def find_ole_in_ppt(filename):
@@ -503,29 +622,35 @@ def find_ole_in_ppt(filename):
     can contain the actual embedded file we are looking for (caller will check
     for these).
     """
-    for stream in PptFile(filename).iter_streams():
-        for record_idx, record in enumerate(stream.iter_records()):
-            if isinstance(record, PptRecordExOleVbaActiveXAtom):
-                ole = None
-                try:
-                    data_start = next(record.iter_uncompressed())
-                    if data_start[:len(olefile.MAGIC)] != olefile.MAGIC:
-                        continue   # could be an ActiveX control or VBA Storage
+    ppt_file = None
+    try:
+        ppt_file = PptFile(filename)
+        for stream in ppt_file.iter_streams():
+            for record_idx, record in enumerate(stream.iter_records()):
+                if isinstance(record, PptRecordExOleVbaActiveXAtom):
+                    ole = None
+                    try:
+                        data_start = next(record.iter_uncompressed())
+                        if data_start[:len(olefile.MAGIC)] != olefile.MAGIC:
+                            continue   # could be ActiveX control / VBA Storage
 
-                    # otherwise, this should be an OLE object
-                    log.debug('Found record with embedded ole object in ppt '
-                              '(stream "{0}", record no {1})'
-                              .format(stream.name, record_idx))
-                    ole = record.get_data_as_olefile()
-                    yield ole
-                except IOError:
-                    log.warning('Error reading data from {0} stream or '
-                                'interpreting it as OLE object'
-                                .format(stream.name))
-                    log.debug('', exc_info=True)
-                finally:
-                    if ole is not None:
-                        ole.close()
+                        # otherwise, this should be an OLE object
+                        log.debug('Found record with embedded ole object in '
+                                  'ppt (stream "{0}", record no {1})'
+                                  .format(stream.name, record_idx))
+                        ole = record.get_data_as_olefile()
+                        yield ole
+                    except IOError:
+                        log.warning('Error reading data from {0} stream or '
+                                    'interpreting it as OLE object'
+                                    .format(stream.name))
+                        log.debug('', exc_info=True)
+                    finally:
+                        if ole is not None:
+                            ole.close()
+    finally:
+        if ppt_file is not None:
+            ppt_file.close()
 
 
 class FakeFile(io.RawIOBase):
@@ -596,7 +721,7 @@ class FakeFile(io.RawIOBase):
         return self.pos
 
 
-def find_ole(filename, data):
+def find_ole(filename, data, xml_parser=None):
     """ try to open somehow as zip/ole/rtf/... ; yield None if fail
 
     If data is given, filename is (mostly) ignored.
@@ -628,34 +753,41 @@ def find_ole(filename, data):
             log.info('is ole file: ' + filename)
             ole = olefile.OleFileIO(arg_for_ole)
             yield ole
-        elif is_zipfile(arg_for_zip):
+        elif xml_parser is not None or is_zipfile(arg_for_zip):
+            # keep compatibility with 3rd-party code that calls this function
+            # directly without providing an XmlParser instance
+            if xml_parser is None:
+                xml_parser = XmlParser(arg_for_zip)
+                # force iteration so XmlParser.iter_non_xml() returns data
+                for _ in xml_parser.iter_xml():
+                    pass
+
             log.info('is zip file: ' + filename)
-            zipper = ZipFile(arg_for_zip, 'r')
-            for subfile in zipper.namelist():
-                head = b''
+            # we looped through the XML files before, now we can
+            # iterate the non-XML files looking for ole objects
+            for subfile, _, file_handle in xml_parser.iter_non_xml():
                 try:
-                    with zipper.open(subfile) as file_handle:
-                        head = file_handle.read(len(olefile.MAGIC))
+                    head = file_handle.read(len(olefile.MAGIC))
                 except RuntimeError:
                     log.error('zip is encrypted: ' + filename)
                     yield None
                     continue
 
                 if head == olefile.MAGIC:
+                    file_handle.seek(0)
                     log.info('  unzipping ole: ' + subfile)
-                    with ZipSubFile(zipper, subfile) as file_handle:
-                        try:
-                            ole = olefile.OleFileIO(file_handle)
-                            yield ole
-                        except IOError:
-                            log.warning('Error reading data from {0}/{1} or '
-                                        'interpreting it as OLE object'
-                                        .format(filename, subfile))
-                            log.debug('', exc_info=True)
-                        finally:
-                            if ole is not None:
-                                ole.close()
-                                ole = None
+                    try:
+                        ole = olefile.OleFileIO(file_handle)
+                        yield ole
+                    except IOError:
+                        log.warning('Error reading data from {0}/{1} or '
+                                    'interpreting it as OLE object'
+                                    .format(filename, subfile))
+                        log.debug('', exc_info=True)
+                    finally:
+                        if ole is not None:
+                            ole.close()
+                            ole = None
                 else:
                     log.debug('unzip skip: ' + subfile)
         else:
@@ -671,6 +803,35 @@ def find_ole(filename, data):
             ole.close()
 
 
+def find_external_relationships(xml_parser):
+    """ iterate XML files looking for relationships to external objects
+    """
+    for _, elem, _ in xml_parser.iter_xml(None, False, OOXML_RELATIONSHIP_TAG):
+        try:
+            if elem.attrib['TargetMode'] == 'External':
+                relationship_type = elem.attrib['Type'].rsplit('/', 1)[1]
+
+                if relationship_type in BLACKLISTED_RELATIONSHIP_TYPES:
+                    yield relationship_type, elem.attrib['Target']
+        except (AttributeError, KeyError):
+            # ignore missing attributes - Word won't detect
+            # external links anyway
+            pass
+
+
+def find_customUI(xml_parser):
+    """
+    iterate XML files looking for customUI to external objects or VBA macros
+    Examples of malicious usage, to load an external document or trigger a VBA macro:
+    https://www.trellix.com/en-us/about/newsroom/stories/threat-labs/prime-ministers-office-compromised.html
+    https://www.netero1010-securitylab.com/evasion/execution-of-remote-vba-script-in-excel
+    """
+    for _, elem, _ in xml_parser.iter_xml(None, False, (TAG_CUSTOMUI_2007, TAG_CUSTOMUI_2010)):
+       customui_onload = elem.get('onLoad')
+       if customui_onload is not None:
+            yield customui_onload
+
+
 def process_file(filename, data, output_dir=None):
     """ find embedded objects in given file
 
@@ -681,16 +842,17 @@ def process_file(filename, data, output_dir=None):
     If output_dir is given and does not exist, it is created. If it is not
     given, data is saved to same directory as the input file.
     """
+    # sanitize filename, leave space for embedded filename part
+    sane_fname = sanitize_filename(filename, max_len=MAX_FILENAME_LENGTH-5) or\
+        'NONAME'
     if output_dir:
         if not os.path.isdir(output_dir):
             log.info('creating output directory %s', output_dir)
             os.mkdir(output_dir)
 
-        fname_prefix = os.path.join(output_dir,
-                                    sanitize_filename(filename))
+        fname_prefix = os.path.join(output_dir, sane_fname)
     else:
         base_dir = os.path.dirname(filename)
-        sane_fname = sanitize_filename(filename)
         fname_prefix = os.path.join(base_dir, sane_fname)
 
     # TODO: option to extract objects to files (false by default)
@@ -703,17 +865,31 @@ def process_file(filename, data, output_dir=None):
     err_dumping = False
     did_dump = False
 
+    xml_parser = None
+    if is_zipfile(filename):
+        log.info('file could be an OOXML file, looking for relationships with '
+                 'external links')
+        xml_parser = XmlParser(filename)
+        for relationship, target in find_external_relationships(xml_parser):
+            did_dump = True
+            print("Found relationship '%s' with external link %s" % (relationship, target))
+            if target.startswith('mhtml:'):
+                print("Potential exploit for CVE-2021-40444")
+        for target in find_customUI(xml_parser):
+            did_dump = True
+            print("Found customUI tag with external link or VBA macro %s (possibly exploiting CVE-2021-42292)" % target)
+
     # look for ole files inside file (e.g. unzip docx)
     # have to finish work on every ole stream inside iteration, since handles
     # are closed in find_ole
-    for ole in find_ole(filename, data):
+    for ole in find_ole(filename, data, xml_parser):
         if ole is None:    # no ole file found
             continue
 
         for path_parts in ole.listdir():
-            if path_parts[-1] == '\x01Ole10Native':
-                stream_path = '/'.join(path_parts)
-                log.debug('Checking stream %r', stream_path)
+            stream_path = '/'.join(path_parts)
+            log.debug('Checking stream %r', stream_path)
+            if path_parts[-1].lower() == '\x01ole10native':
                 stream = None
                 try:
                     stream = ole.openstream(path_parts)
@@ -737,11 +913,12 @@ def process_file(filename, data, output_dir=None):
                 print(u'Filename = "%s"' % opkg.filename)
                 print(u'Source path = "%s"' % opkg.src_path)
                 print(u'Temp path = "%s"' % opkg.temp_path)
-                if opkg.filename:
-                    fname = '%s_%s' % (fname_prefix,
-                                       sanitize_filename(opkg.filename))
-                else:
-                    fname = '%s_object_%03d.noname' % (fname_prefix, index)
+                for embedded_fname in get_sane_embedded_filenames(
+                        opkg.filename, opkg.src_path, opkg.temp_path,
+                        MAX_FILENAME_LENGTH - len(sane_fname) - 1, index):
+                    fname = fname_prefix + '_' + embedded_fname
+                    if not os.path.isfile(fname):
+                        break
 
                 # dump
                 try:
@@ -788,6 +965,7 @@ def main(cmd_line_args=None):
     provide other arguments.
     """
     # print banner with version
+    ensure_stdout_handles_unicode()
     print('oleobj %s - http://decalage.info/oletools' % __version__)
     print('THIS IS WORK IN PROGRESS - Check updates regularly!')
     print('Please report any issue at '
