@@ -1,20 +1,64 @@
-#!/usr/local/bin/python2.7
+#!/usr/bin/env python3
 
-""" Common operations for OpenOffice XML (docx, xlsx, pptx, ...) files
+""" Common operations for OpenXML files (docx, xlsx, pptx, ...)
 
 This is mostly based on ECMA-376 (5th edition, Part 1)
 http://www.ecma-international.org/publications/standards/Ecma-376.htm
 
 See also: Notes on Microsoft's implementation of ECMA-376: [MS-0E376]
 
-TODO: may have to tell apart single xml types: office2003 looks much different
-      than 2006+ --> DOCTYPE_*_XML2003
-
 .. codeauthor:: Intra2net AG <info@intra2net>
+License: BSD, see source code or documentation
+
+ooxml is part of the python-oletools package:
+http://www.decalage.info/python/oletools
 """
 
+# === LICENSE =================================================================
+
+# ooxml is copyright (c) 2017-2020 Philippe Lagadec (http://www.decalage.info)
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+#  * Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+
+# -----------------------------------------------------------------------------
+# CHANGELOG:
+# 2018-12-06       CH: - ensure stdout can handle unicode
+
+__version__ = '0.54.2'
+
+# -- TODO ---------------------------------------------------------------------
+
+# TODO: may have to tell apart single xml types: office2003 looks much different
+#       than 2006+ --> DOCTYPE_*_XML2003
+# TODO: check what is duplicate here with oleid, maybe merge some day?
+# TODO: "xml2003" == "flatopc"? (No)
+
+
+# -- IMPORTS ------------------------------------------------------------------
+
 import sys
-import logging
+from oletools.common.log_helper import log_helper
+from oletools.common.io_encoding import uopen
 from zipfile import ZipFile, BadZipfile, is_zipfile
 from os.path import splitext
 import io
@@ -27,6 +71,12 @@ try:
 except ImportError:
     import xml.etree.cElementTree as ET
 
+###############################################################################
+# CONSTANTS
+###############################################################################
+
+
+logger = log_helper.get_or_create_silent_logger('ooxml')
 
 #: subfiles that have to be part of every ooxml file
 FILE_CONTENT_TYPES = '[Content_Types].xml'
@@ -104,16 +154,14 @@ def debug_str(elem):
     text = u', '.join(parts)
     if len(text) > 150:
         return text[:147] + u'...]'
-    else:
-        return text + u']'
+    return text + u']'
 
 
 def isstr(some_var):
     """ version-independent test for isinstance(some_var, (str, unicode)) """
     if sys.version_info.major == 2:
         return isinstance(some_var, basestring)  # true for str and unicode
-    else:
-        return isinstance(some_var, str)         # there is no unicode
+    return isinstance(some_var, str)         # there is no unicode
 
 
 ###############################################################################
@@ -126,30 +174,36 @@ def get_type(filename):
     parser = XmlParser(filename)
     if parser.is_single_xml():
         match = None
-        with open(filename, 'r') as handle:
+        with uopen(filename, 'r') as handle:
             match = re.search(OFFICE_XML_PROGID_REGEX, handle.read(1024))
         if not match:
             return DOCTYPE_NONE
         prog_id = match.groups()[0]
         if prog_id == WORD_XML_PROG_ID:
             return DOCTYPE_WORD_XML
-        elif prog_id == EXCEL_XML_PROG_ID:
+        if prog_id == EXCEL_XML_PROG_ID:
             return DOCTYPE_EXCEL_XML
-        else:
-            return DOCTYPE_NONE
+        return DOCTYPE_NONE
 
     is_doc = False
     is_xls = False
     is_ppt = False
-    for _, elem, _ in parser.iter_xml(FILE_CONTENT_TYPES):
-        logging.debug(u'  ' + debug_str(elem))
-        try:
-            content_type = elem.attrib['ContentType']
-        except KeyError:         # ContentType not an attr
-            continue
-        is_xls |= content_type.startswith(CONTENT_TYPES_EXCEL)
-        is_doc |= content_type.startswith(CONTENT_TYPES_WORD)
-        is_ppt |= content_type.startswith(CONTENT_TYPES_PPT)
+    try:
+        for _, elem, _ in parser.iter_xml(FILE_CONTENT_TYPES):
+            logger.debug(u'  ' + debug_str(elem))
+            try:
+                content_type = elem.attrib['ContentType']
+            except KeyError:         # ContentType not an attr
+                continue
+            is_xls |= content_type.startswith(CONTENT_TYPES_EXCEL)
+            is_doc |= content_type.startswith(CONTENT_TYPES_WORD)
+            is_ppt |= content_type.startswith(CONTENT_TYPES_PPT)
+    except BadOOXML as oo_err:
+        if oo_err.more_info.startswith('invalid subfile') and \
+                FILE_CONTENT_TYPES in oo_err.more_info:
+            # no FILE_CONTENT_TYPES in zip, so probably no ms office xml.
+            return DOCTYPE_NONE
+        raise
 
     if is_doc and not is_xls and not is_ppt:
         return DOCTYPE_WORD
@@ -159,9 +213,8 @@ def get_type(filename):
         return DOCTYPE_POWERPOINT
     if not is_doc and not is_xls and not is_ppt:
         return DOCTYPE_NONE
-    else:
-        logging.warning('Encountered contradictory content types')
-        return DOCTYPE_MIXED
+    logger.warning('Encountered contradictory content types')
+    return DOCTYPE_MIXED
 
 
 def is_ooxml(filename):
@@ -174,6 +227,7 @@ def is_ooxml(filename):
         return False
     if doctype == DOCTYPE_NONE:
         return False
+    return True
 
 
 ###############################################################################
@@ -213,6 +267,7 @@ class ZipSubFile(object):
     See also (and maybe could some day merge with):
     ppt_record_parser.IterStream; also: oleobj.FakeFile
     """
+    CHUNK_SIZE = 4096
 
     def __init__(self, container, filename, mode='r', size=None):
         """ remember all necessary vars but do not open yet """
@@ -220,7 +275,7 @@ class ZipSubFile(object):
         self.name = filename
         if size is None:
             self.size = container.getinfo(filename).file_size
-            logging.debug('zip stream has size {0}'.format(self.size))
+            logger.debug('zip stream has size {0}'.format(self.size))
         else:
             self.size = size
         if 'w' in mode.lower():
@@ -250,7 +305,7 @@ class ZipSubFile(object):
         # print('ZipSubFile: opened; size={}'.format(self.size))
         return self
 
-    def write(self, *args, **kwargs):                          # pylint: disable=unused-argument,no-self-use
+    def write(self, *args, **kwargs):
         """ write is not allowed """
         raise IOError('writing not implemented')
 
@@ -308,10 +363,9 @@ class ZipSubFile(object):
         """ helper for seek: skip forward by given amount using read() """
         # print('ZipSubFile: seek by skipping {} bytes starting at {}'
         #       .format(self.pos, to_skip))
-        CHUNK_SIZE = 4096
-        n_chunks, leftover = divmod(to_skip, CHUNK_SIZE)
+        n_chunks, leftover = divmod(to_skip, self.CHUNK_SIZE)
         for _ in range(n_chunks):
-            self.read(CHUNK_SIZE)    # just read and discard
+            self.read(self.CHUNK_SIZE)    # just read and discard
         self.read(leftover)
         # print('ZipSubFile: seek by skipping done, pos now {}'
         #       .format(self.pos))
@@ -409,20 +463,26 @@ class XmlParser(object):
 
         # find prog id in xml prolog
         match = None
-        with open(self.filename, 'r') as handle:
+        with uopen(self.filename, 'r') as handle:
             match = re.search(OFFICE_XML_PROGID_REGEX, handle.read(1024))
         if match:
             self._is_single_xml = True
             return True
-        if not match:
-            raise BadOOXML(self.filename, 'is no zip and has no prog_id')
+        raise BadOOXML(self.filename, 'is no zip and has no prog_id')
 
     def iter_files(self, args=None):
-        """ Find files in zip or just give single xml file """
+        """
+        Find files in zip or just give single xml file
+
+        yields pairs (subfile-name, file-handle) where file-handle is an open
+        file-like object. (Do not care too much about encoding here, the xml
+        parser reads the encoding from the first lines in the file.)
+        """
         if self.is_single_xml():
             if args:
                 raise BadOOXML(self.filename, 'xml has no subfiles')
-            with open(self.filename, 'r') as handle:
+            # do not use uopen, xml parser determines encoding on its own
+            with open(self.filename, 'rb') as handle:
                 yield None, handle   # the subfile=None is needed in iter_xml
             self.did_iter_all = True
         else:
@@ -430,17 +490,14 @@ class XmlParser(object):
             subfiles = None
             try:
                 zipper = ZipFile(self.filename)
-                try:
-                    _ = zipper.getinfo(FILE_CONTENT_TYPES)
-                except KeyError:
-                    raise BadOOXML(self.filename,
-                                   'No content type information')
                 if not args:
                     subfiles = zipper.namelist()
                 elif isstr(args):
                     subfiles = [args, ]
                 else:
-                    subfiles = tuple(args)   # make a copy in case orig changes
+                    # make a copy in case original args are modified
+                    # Not sure whether this really is needed...
+                    subfiles = tuple(arg for arg in args)
 
                 for subfile in subfiles:
                     with zipper.open(subfile, 'r') as handle:
@@ -448,10 +505,12 @@ class XmlParser(object):
                 if not args:
                     self.did_iter_all = True
             except KeyError as orig_err:
+                # Note: do not change text of this message without adjusting
+                #       conditions in except handlers
                 raise BadOOXML(self.filename,
                                'invalid subfile: ' + str(orig_err))
             except BadZipfile:
-                raise BadOOXML(self.filename, 'neither zip nor xml')
+                raise BadOOXML(self.filename, 'not in zip format')
             finally:
                 if zipper:
                     zipper.close()
@@ -484,10 +543,10 @@ class XmlParser(object):
             want_tags = []
         elif isstr(tags):
             want_tags = [tags, ]
-            logging.debug('looking for tags: {0}'.format(tags))
+            logger.debug('looking for tags: {0}'.format(tags))
         else:
             want_tags = tags
-            logging.debug('looking for tags: {0}'.format(tags))
+            logger.debug('looking for tags: {0}'.format(tags))
 
         for subfile, handle in self.iter_files(subfiles):
             events = ('start', 'end')
@@ -499,8 +558,8 @@ class XmlParser(object):
                         continue
                     if event == 'start':
                         if elem.tag in want_tags:
-                            logging.debug('remember start of tag {0} at {1}'
-                                          .format(elem.tag, depth))
+                            logger.debug('remember start of tag {0} at {1}'
+                                         .format(elem.tag, depth))
                             inside_tags.append((elem.tag, depth))
                         depth += 1
                         continue
@@ -515,19 +574,19 @@ class XmlParser(object):
                             if inside_tags[-1] == curr_tag:
                                 inside_tags.pop()
                             else:
-                                logging.error('found end for wanted tag {0} '
-                                              'but last start tag {1} does not'
-                                              ' match'.format(curr_tag,
-                                                              inside_tags[-1]))
+                                logger.error('found end for wanted tag {0} '
+                                             'but last start tag {1} does not'
+                                             ' match'.format(curr_tag,
+                                                             inside_tags[-1]))
                                 # try to recover: close all deeper tags
                                 while inside_tags and \
                                         inside_tags[-1][1] >= depth:
-                                    logging.debug('recover: pop {0}'
-                                                  .format(inside_tags[-1]))
+                                    logger.debug('recover: pop {0}'
+                                                 .format(inside_tags[-1]))
                                     inside_tags.pop()
                         except IndexError:    # no inside_tag[-1]
-                            logging.error('found end of {0} at depth {1} but '
-                                          'no start event')
+                            logger.error('found end of {0} at depth {1} but '
+                                         'no start event')
                     # yield element
                     if is_wanted or not want_tags:
                         yield subfile, elem, depth
@@ -541,14 +600,14 @@ class XmlParser(object):
             except ET.ParseError as err:
                 self.subfiles_no_xml.add(subfile)
                 if subfile is None:    # this is no zip subfile but single xml
-                    raise BadOOXML(self.filename, 'is neither zip nor xml')
+                    raise BadOOXML(self.filename, 'content is not valid XML')
                 elif subfile.endswith('.xml'):
-                    logger = logging.warning
+                    log = logger.warning
                 else:
-                    logger = logging.debug
-                logger('  xml-parsing for {0} failed ({1}). '
-                       .format(subfile, err) +
-                       'Run iter_non_xml to investigate.')
+                    log = logger.debug
+                log('  xml-parsing for {0} failed ({1}). '
+                    .format(subfile, err) +
+                    'Run iter_non_xml to investigate.')
             assert(depth == 0)
 
     def get_content_types(self):
@@ -565,21 +624,30 @@ class XmlParser(object):
 
         defaults = []
         files = []
-        for _, elem, _ in self.iter_xml(FILE_CONTENT_TYPES):
-            if elem.tag.endswith('Default'):
-                extension = elem.attrib['Extension']
-                if extension.startswith('.'):
-                    extension = extension[1:]
-                defaults.append((extension, elem.attrib['ContentType']))
-                logging.debug('found content type for extension {0[0]}: {0[1]}'
-                              .format(defaults[-1]))
-            elif elem.tag.endswith('Override'):
-                subfile = elem.attrib['PartName']
-                if subfile.startswith('/'):
-                    subfile = subfile[1:]
-                files.append((subfile, elem.attrib['ContentType']))
-                logging.debug('found content type for subfile {0[0]}: {0[1]}'
-                              .format(files[-1]))
+        try:
+            for _, elem, _ in self.iter_xml(FILE_CONTENT_TYPES):
+                if elem.tag.endswith('Default'):
+                    extension = elem.attrib['Extension']
+                    if extension.startswith('.'):
+                        extension = extension[1:]
+                    defaults.append((extension, elem.attrib['ContentType']))
+                    logger.debug('found content type for extension {0[0]}: '
+                                 '{0[1]}'.format(defaults[-1]))
+                elif elem.tag.endswith('Override'):
+                    subfile = elem.attrib['PartName']
+                    if subfile.startswith('/'):
+                        subfile = subfile[1:]
+                    files.append((subfile, elem.attrib['ContentType']))
+                    logger.debug('found content type for subfile {0[0]}: '
+                                 '{0[1]}'.format(files[-1]))
+        except BadOOXML as oo_err:
+            if oo_err.more_info.startswith('invalid subfile') and \
+                    FILE_CONTENT_TYPES in oo_err.more_info:
+                # no FILE_CONTENT_TYPES in zip, so probably no ms office xml.
+                # Maybe OpenDocument format? In any case, try to analyze.
+                pass
+            else:
+                raise
         return dict(files), dict(defaults)
 
     def iter_non_xml(self):
@@ -595,14 +663,14 @@ class XmlParser(object):
         To handle binary parts of an xlsb file, use xls_parser.parse_xlsb_part
         """
         if not self.did_iter_all:
-            logging.warning('Did not iterate through complete file. '
-                            'Should run iter_xml() without args, first.')
+            logger.warning('Did not iterate through complete file. '
+                           'Should run iter_xml() without args, first.')
         if not self.subfiles_no_xml:
-            raise StopIteration()
+            return
 
         # case of single xml files (office 2003+)
         if self.is_single_xml():
-            raise StopIteration()   # "return"
+            return
 
         content_types, content_defaults = self.get_content_types()
 
@@ -624,11 +692,12 @@ class XmlParser(object):
 
 
 def test():
-    """ Main function, called when running file as script
-
-    see module doc for more info
     """
-    logging.basicConfig(level=logging.DEBUG)
+    Test xml parsing; called when running this file as a script.
+
+    Prints every element found in input file (to be given as command line arg).
+    """
+    log_helper.enable_logging(False, 'debug')
     if len(sys.argv) != 2:
         print(u'To test this code, give me a single file as arg')
         return 2
@@ -641,12 +710,15 @@ def test():
     for subfile, elem, depth in parser.iter_xml():
         if depth < 4:
             print(u'{0} {1}{2}'.format(subfile, '  ' * depth, debug_str(elem)))
-    for index, (subfile, content_type) in enumerate(parser.iter_non_xml()):
+    for index, (subfile, content_type, _) in enumerate(parser.iter_non_xml()):
         print(u'Non-XML subfile: {0} of type {1}'
               .format(subfile, content_type or u'unknown'))
         if index > 100:
             print(u'...')
             break
+
+    log_helper.end_logging()
+
     return 0
 
 
